@@ -20,14 +20,15 @@ from datetime import datetime, timedelta
 
 from mrack.errors import ProvisioningError, ValidationError
 from mrack.host import STATUS_ACTIVE, STATUS_OTHER, Host
-from mrack.utils import ssh_to_host
+from mrack.utils import global_context, ssh_to_host
 
 logger = logging.getLogger(__name__)
 
 STRATEGY_ABORT = "abort"
 STRATEGY_RETRY = "retry"
-RET_CODE = 0
-HOST_OBJ = 1
+RET_CODE = 0  # index to access return code from _wait_for_ssh
+HOST_OBJ = 1  # index to access host object from _wait_for_ssh
+SPECS = 1  # default index to access host specs which caused ProvisioningError
 
 
 class Provider:
@@ -127,10 +128,16 @@ class Provider:
 
         return res, host
 
-    async def _provision_base(self, reqs):  # pylint: disable=too-many-locals
+    async def _provision_base(
+        self, reqs, res_check_timeout=60, res_busy_sleep=10
+    ):  # pylint: disable=too-many-locals, too-many-branches
         """Provision hosts based on list of host requirements.
 
-        Main function which does provisioning and not any validation.
+        Main function which does provisioning and validation.
+        Parameters:
+            reqs - dictionary with requirements for provider
+            res_check_timeout (default 60) - timeout (minutes) to wait for resources
+            res_busy_sleep (default 10) - time to wait before checking again (minutes)
         """
         logger.info(f"{self.dsp_name}: Validating hosts definitions")
         if not reqs:
@@ -143,8 +150,13 @@ class Provider:
 
         logger.info(f"{self.dsp_name}: Checking available resources")
 
-        if not await self.can_provision(reqs):
-            raise ValidationError(f"{self.dsp_name}: Not enough resources to provision")
+        res_check_start = datetime.now()
+        while not await self.can_provision(reqs):
+            await asyncio.sleep(res_busy_sleep * 60)
+            if datetime.now() - res_check_start >= timedelta(minutes=res_check_timeout):
+                raise ValidationError(
+                    f"{self.dsp_name}: Not enough resources to provision"
+                )
         logger.info(f"{self.dsp_name}: Resource availability: OK")
         started = datetime.now()
 
@@ -153,14 +165,35 @@ class Provider:
         for req in reqs:
             awaitable = self.create_server(req)
             create_servers.append(awaitable)
-        create_resps = await asyncio.gather(*create_servers)
+
+        # expect the exception in return data to be parsed later
+        create_resps = await asyncio.gather(*create_servers, return_exceptions=True)
+
         logger.info(f"{self.dsp_name}: Provisioning issued")
 
         logger.info(f"{self.dsp_name}: Waiting for all hosts to be active")
+
+        error_hosts = []
         wait_servers = []
-        for create_resp in create_resps:
-            awaitable = self.wait_till_provisioned(create_resp)
-            wait_servers.append(awaitable)
+        for response in create_resps:
+            if not isinstance(response, ProvisioningError):
+                # response might be okay so let us wait for result
+                awaitable = self.wait_till_provisioned(response)
+                wait_servers.append(awaitable)
+            else:
+                # use ProvisioningError arguments to create missing Host object
+                # which we append to error hosts list for later usage
+                error_hosts.append(
+                    Host(
+                        provider=self,
+                        host_id=None,
+                        name=response.args[SPECS]["name"],
+                        ip_addrs=[],
+                        status=STATUS_OTHER,
+                        rawdata=response.args,
+                        error_obj=response.args,
+                    )
+                )
 
         server_results = await asyncio.gather(*wait_servers)
         provisioned = datetime.now()
@@ -171,33 +204,41 @@ class Provider:
         )
         logger.info(f"{self.dsp_name}: Provisioning duration: {provisioned - started}")
 
-        hosts = [self.to_host(srv) for srv in server_results]
-        error_hosts = await self.parse_error_hosts(hosts)
-        active_hosts = [h for h in hosts if h not in error_hosts]
-        # check ssh connectivity to succeeded hosts
-        wait_ssh = []
-        for host in active_hosts:
-            awaitable = self._wait_for_ssh(host)
-            wait_ssh.append(awaitable)
+        hosts = [self.to_host(srv) for srv in server_results if srv]
 
-        ssh_results = await asyncio.gather(*wait_ssh)
-        # We distinguish the success hosts and new error hosts from active by using:
-        # res[RET_CODE] 0 - the result of operation returned from self._wait_for_ssh()
-        # res[HOST_OBJ] 1 - the host object returned from self._wait_for_ssh()
+        error_hosts += await self.parse_error_hosts(hosts)
+        active_hosts = [h for h in hosts if h not in error_hosts]
         success_hosts = []
-        for res in ssh_results:
-            if res[RET_CODE]:
-                success_hosts.append(res[HOST_OBJ])
-            else:
-                res[HOST_OBJ].error = (
-                    "Could not establish ssh connection to host "
-                    f"{res[HOST_OBJ].host_id} with IP {res[HOST_OBJ].ip_addr}"
-                )
-                error_hosts.append(res[HOST_OBJ])
+
+        if global_context["config"].get("post_provisioning_ssh_check", True):
+            # check ssh connectivity to succeeded hosts
+            wait_ssh = []
+            for host in active_hosts:
+                awaitable = self._wait_for_ssh(host)
+                wait_ssh.append(awaitable)
+
+            ssh_results = await asyncio.gather(*wait_ssh)
+            # We distinguish the success hosts and new error hosts from active by using:
+            # res[RET_CODE] 0
+            #   - the result of operation returned from self._wait_for_ssh()
+            # res[HOST_OBJ] 1
+            #   - the host object returned from self._wait_for_ssh()
+            for res in ssh_results:
+                if res[RET_CODE]:
+                    success_hosts.append(res[HOST_OBJ])
+                else:
+                    res[HOST_OBJ].error = (
+                        "Could not establish ssh connection to host "
+                        f"{res[HOST_OBJ].host_id} with IP {res[HOST_OBJ].ip_addr}"
+                    )
+                    error_hosts.append(res[HOST_OBJ])
+        else:  # we do not check the ssh connection to VMs
+            success_hosts = active_hosts
 
         missing_reqs = [
             req for req in reqs if req["name"] in [host.name for host in error_hosts]
         ]
+
         return (success_hosts, error_hosts, missing_reqs)
 
     async def provision_hosts(self, reqs):
@@ -205,10 +246,7 @@ class Provider:
 
         Main provider method for provisioning.
 
-        First it validates that host requirements are valid and that
-        provider has enough resources(quota).
-
-        Then issues provisioning and waits for it succeed. Raises exception if any of
+        Issues provisioning and waits for it succeed. Raises exception if any of
         the servers was not successfully provisioned. If that happens it issues deletion
         of all already provisioned resources.
 
@@ -217,9 +255,6 @@ class Provider:
         logger.info(f"{self.dsp_name}: Preparing provider resources")
         await self.prepare_provisioning(reqs)
 
-        logger.info(f"{self.dsp_name}: Validating hosts definitions")
-        await self.validate_hosts(reqs)
-
         if self.strategy == STRATEGY_RETRY:
             success_hosts, error_hosts, _missing_reqs = await self.strategy_retry(reqs)
         else:
@@ -324,6 +359,7 @@ class Provider:
             self.status_map.get(host_info.get("status"), STATUS_OTHER),
             provisioning_result,
             username=username,
+            password=host_info.get("password"),
             error_obj=host_info.get("fault"),
         )
         return host
